'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _actions = require('../actions');

var describeSyncErrors = function describeSyncErrors(reducer, expect, _ref) {
  var fromJS = _ref.fromJS;
  var getIn = _ref.getIn;
  return function () {
    it('should validate on init with no state', function () {
      var errors = { bar: 'Required' };
      var state = reducer.validation({
        foo: function foo() {
          return errors;
        }
      })();
      expect(state).toEqualMap({
        foo: {
          syncErrors: errors
        }
      });
    });

    it('should validate on init with state', function () {
      var state = reducer.validation({
        foo: function foo(values) {
          return {
            bar: 'Was not expecting "' + getIn(values, 'bar') + '"'
          };
        }
      })(fromJS({
        foo: {
          values: {
            bar: 'the Spanish Inquisition'
          }
        }
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            bar: 'the Spanish Inquisition'
          },
          syncErrors: {
            bar: 'Was not expecting "the Spanish Inquisition"'
          }
        }
      });
    });

    it('should call validate with {} when no values', function () {
      reducer.validation({
        foo: function foo(values) {
          return expect(values).toEqualMap({});
        }
      })();
    });

    it('should validate on blur', function () {
      var validatingReducer = reducer.validation({
        foo: function foo(values) {
          var bar = getIn(values, 'bar');
          if (bar === undefined) {
            return { bar: 'Required' };
          }
          if (bar.length > 5) {
            return { bar: 'Too long' };
          }
          return {};
        }
      });

      var requiredState = validatingReducer();
      expect(requiredState).toEqualMap({
        foo: {
          syncErrors: {
            bar: 'Required'
          }
        }
      });

      var validState = validatingReducer(requiredState, _extends({}, (0, _actions.blur)('dog'), {
        form: 'foo',
        field: 'bar'
      }));
      expect(validState).toEqualMap({
        foo: {
          values: {
            bar: 'dog'
          }
        }
      });

      var tooLongState = validatingReducer(validState, _extends({}, (0, _actions.blur)('rabbit'), {
        form: 'foo',
        field: 'bar'
      }));
      expect(tooLongState).toEqualMap({
        foo: {
          values: {
            bar: 'rabbit'
          },
          syncErrors: {
            bar: 'Too long'
          }
        }
      });
    });

    it('should validate on change', function () {
      var validatingReducer = reducer.validation({
        foo: function foo(values) {
          var bar = getIn(values, 'bar');
          if (bar === undefined) {
            return { bar: 'Required' };
          }
          if (bar.length > 5) {
            return { bar: 'Too long' };
          }
          return {};
        }
      });

      var requiredState = validatingReducer();
      expect(requiredState).toEqualMap({
        foo: {
          syncErrors: {
            bar: 'Required'
          }
        }
      });

      var validState = validatingReducer(requiredState, _extends({}, (0, _actions.change)('dog'), {
        form: 'foo',
        field: 'bar'
      }));
      expect(validState).toEqualMap({
        foo: {
          values: {
            bar: 'dog'
          }
        }
      });

      var tooLongState = validatingReducer(validState, _extends({}, (0, _actions.change)('rabbit'), {
        form: 'foo',
        field: 'bar'
      }));
      expect(tooLongState).toEqualMap({
        foo: {
          values: {
            bar: 'rabbit'
          },
          syncErrors: {
            bar: 'Too long'
          }
        }
      });
    });

    it('should set sync error on nested fields', function () {
      var validatingReducer = reducer.validation({
        foo: function foo(values) {
          expect(values).toEqualMap({
            bar: {
              myField: 'dirtyValue',
              myOtherField: 'otherDirtyValue'
            }
          });
          return {
            bar: {
              myField: 'Error about myField',
              myOtherField: 'Error about myOtherField'
            }
          };
        }
      });
      var state = validatingReducer(fromJS({
        foo: {
          values: {
            bar: {
              myField: 'dirtyValue',
              myOtherField: 'otherDirtyValue'
            }
          },
          initial: {
            bar: {
              myField: 'initialValue',
              myOtherField: 'otherInitialValue'
            }
          },
          fields: {
            bar: {
              myField: {
                touched: true
              },
              myOtherField: {
                touched: true
              }
            }
          }
        }
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            bar: {
              myField: 'dirtyValue',
              myOtherField: 'otherDirtyValue'
            }
          },
          initial: {
            bar: {
              myField: 'initialValue',
              myOtherField: 'otherInitialValue'
            }
          },
          syncErrors: {
            bar: {
              myField: 'Error about myField',
              myOtherField: 'Error about myOtherField'
            }
          },
          fields: {
            bar: {
              myField: {
                touched: true
              },
              myOtherField: {
                touched: true
              }
            }
          }
        }
      });
    });

    it('should set sync errors on array fields', function () {
      var validatingReducer = reducer.validation({
        foo: function foo(values) {
          expect(values).toEqualMap({
            bar: ['dirtyValue', 'otherDirtyValue']
          });
          return {
            bar: ['Error about myField', 'Error about myOtherField']
          };
        }
      });
      var state = validatingReducer(fromJS({
        foo: {
          values: {
            bar: ['dirtyValue', 'otherDirtyValue']
          },
          initial: {
            bar: ['initialValue', 'otherInitialValue']
          },
          fields: {
            bar: [{
              touched: true
            }, {
              touched: true
            }]
          }
        }
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            bar: ['dirtyValue', 'otherDirtyValue']
          },
          initial: {
            bar: ['initialValue', 'otherInitialValue']
          },
          syncErrors: {
            bar: ['Error about myField', 'Error about myOtherField']
          },
          fields: {
            bar: [{
              touched: true
            }, {
              touched: true
            }]
          }
        }
      });
    });

    it('should allow multiple errors on same field', function () {
      var validatingReducer = reducer.validation({
        foo: function foo(values) {
          expect(values).toEqualMap({
            cat: 'Garfield',
            dog: 'Odie'
          });
          return {
            cat: ['Not funny', 'Sleeps too much'],
            dog: ['Unhelpful', 'Not house trained']
          };
        }
      });
      var state = validatingReducer(fromJS({
        foo: {
          values: {
            cat: 'Garfield',
            dog: 'Odie'
          },
          fields: {
            cat: {
              touched: true
            },
            dog: {
              touched: true
            }
          }
        }
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            cat: 'Garfield',
            dog: 'Odie'
          },
          syncErrors: {
            cat: ['Not funny', 'Sleeps too much'],
            dog: ['Unhelpful', 'Not house trained']
          },
          fields: {
            cat: {
              touched: true
            },
            dog: {
              touched: true
            }
          }
        }
      });
    });

    it('should unset field sync errors', function () {
      var validatingReducer = reducer.validation({
        foo: function foo(values) {
          expect(values).toEqualMap({
            bar: ['dirtyValue', 'otherDirtyValue']
          });
          return {};
        }
      });
      var state = validatingReducer(fromJS({
        foo: {
          values: {
            bar: ['dirtyValue', 'otherDirtyValue']
          },
          initial: {
            bar: ['initialValue', 'otherInitialValue']
          },
          syncErrors: {
            bar: ['sync error 1', 'sync error 2']
          },
          fields: {
            bar: [{
              touched: true
            }, {
              touched: true
            }]
          }
        }
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            bar: ['dirtyValue', 'otherDirtyValue']
          },
          initial: {
            bar: ['initialValue', 'otherInitialValue']
          },
          fields: {
            bar: [{
              touched: true
            }, {
              touched: true
            }]
          }
        }
      });
    });

    it('should unset global errors', function () {
      var validatingReducer = reducer.validation({
        foo: function foo(values) {
          expect(values).toEqualMap({ myField: 'myValue' });
          return {};
        }
      });
      var state = validatingReducer(fromJS({
        foo: {
          values: {
            myField: 'myValue'
          },
          fields: {
            myField: {
              touched: true
            }
          },
          error: 'Previous global error'
        }
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myField: 'myValue'
          },
          fields: {
            myField: {
              touched: true
            }
          }
        }
      });
    });

    it('should set global error', function () {
      var validatingReducer = reducer.validation({
        foo: function foo(values) {
          expect(values).toEqualMap({ myField: 'value' });
          return {
            _error: 'This is a global error'
          };
        }
      });
      var state = validatingReducer(fromJS({
        foo: {
          values: {
            myField: 'value'
          },
          fields: {
            myField: {
              touched: true
            }
          }
        }
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myField: 'value'
          },
          fields: {
            myField: {
              touched: true
            }
          },
          error: 'This is a global error'
        }
      });
    });
  };
};

exports.default = describeSyncErrors;